@@ -5,6 +5,7 @@ from typing import (
     Dict,
     Iterable,
     List,
+    Literal,
     Optional,
     Sequence,
     Set,
@@ -39,6 +40,8 @@ __all__ = [
     "FewShotJinjaMapper",
 ]
 
+VARSHOTS = "__shots__"
+
 
 @Necessary(
     "promptsource",
@@ -112,6 +115,10 @@ class PromptsourceMixin(ChainableMapperMixIn):
             input_fields=input_fields, output_fields=output_fields
         )
 
+    @staticmethod
+    def get_vars_from_txt(text: str) -> Set[str]:
+        return meta.find_undeclared_variables(Environment().parse(text))
+
     @property
     def approx_input_fields(self) -> Tuple[Set[str], ...]:
         """A tuple of sets of input fields that are required by the
@@ -129,9 +136,7 @@ class PromptsourceMixin(ChainableMapperMixIn):
         return tuple(
             set(
                 field
-                for field in meta.find_undeclared_variables(
-                    Environment().parse(t)
-                )
+                for field in self.get_vars_from_txt(t)
                 if field not in self.extra_vars
             )
             for t in self.template.jinja.split("|||")
@@ -350,7 +355,7 @@ class FewShotJinjaMapper(PromptsourceMixin, BatchedBaseMapper):
     def __init__(
         self,
         jinja: str,
-        num_shots: int,
+        num_shots: Union[int, Literal["max"]],
         name: Optional[str] = None,
         reference: Optional[str] = None,
         metadata: Optional["Template.Metadata"] = None,
@@ -374,7 +379,9 @@ class FewShotJinjaMapper(PromptsourceMixin, BatchedBaseMapper):
                 are available as variables in the template. A special
                 variable __shots__ is available, which contains all the shots
                 for the sample.
-            num_shots (int): the number of shots to generate for each sample.
+            num_shots (Union[int, Literal['max']]): the number of samples to
+                use for each sample. If set to 'max', then all the samples
+                in the dataset are used.
             name (Optional[str], optional): the name of the template. Defaults
                 to None.
             reference (Optional[str], optional): the reference ID for the
@@ -404,15 +411,17 @@ class FewShotJinjaMapper(PromptsourceMixin, BatchedBaseMapper):
                 of extra variables that will be passed to the promptsource
                 template. Defaults to None.
         """
-        if not isinstance(num_shots, int) and num_shots >= 0:
+        if num_shots != "max" and not (
+            isinstance(num_shots, int) and num_shots >= 0
+        ):
             raise ValueError(
-                "number_of_shots must be a non-negative integer, "
+                "number_of_shots must be a non-negative integer or 'max', "
                 f"but got {num_shots}"
             )
 
-        if not re.search(r"\b__shots__\b", jinja):
-            raise ValueError(
-                "the jinja template must contain the variable __shots__"
+        if VARSHOTS not in self.get_vars_from_txt(jinja):
+            raise KeyError(
+                f"the jinja template must contain the variable {VARSHOTS}"
             )
 
         template = Template(
@@ -422,8 +431,12 @@ class FewShotJinjaMapper(PromptsourceMixin, BatchedBaseMapper):
             metadata=metadata,
         )
 
-        self.num_shots = num_shots
-        self.keep_last = keep_last
+        # mypy complains if we don't retype num_shots
+        self.num_shots: Union[int, Literal["max"]] = num_shots
+
+        # due to how "max" works, we always need to keep the batch
+        # when in "max" mode, otherwise we will return an empty dataset
+        self.keep_last: bool = keep_last or num_shots == "max"
 
         super().__init__(
             template=template,
@@ -438,7 +451,7 @@ class FewShotJinjaMapper(PromptsourceMixin, BatchedBaseMapper):
     @property
     def approx_input_fields(self) -> Tuple[Set[str], ...]:
         return tuple(
-            set(f for f in fields if f != "__shots__")
+            set(f for f in fields if f != VARSHOTS)
             for fields in super().approx_input_fields
         )
 
@@ -449,12 +462,10 @@ class FewShotJinjaMapper(PromptsourceMixin, BatchedBaseMapper):
         accumulator: List[TransformElementType] = []
 
         for sample in data:
-            if len(accumulator) < self.num_shots:
+            if self.num_shots == "max" or len(accumulator) < self.num_shots:
                 accumulator.append(sample)
             else:
-                output = self.apply_template(
-                    {**sample, "__shots__": accumulator}
-                )
+                output = self.apply_template({**sample, VARSHOTS: accumulator})
                 accumulator = []
                 yield self.format_output(output)
 
@@ -465,5 +476,5 @@ class FewShotJinjaMapper(PromptsourceMixin, BatchedBaseMapper):
             # use the last as the non-context sample
             *accumulator, sample = accumulator
 
-            output = self.apply_template({**sample, "__shots__": accumulator})
+            output = self.apply_template({**sample, VARSHOTS: accumulator})
             yield self.format_output(output)
